function [features, rcfd, maxBright, minBright, meanFeature, V, allOtherData] = make_features_from_scratch(folder, pixelSize, csvFile, imageFileExtension, algorithm, ROI, csvNumCols, csvXCol, csvYCol, csvFCol, pixelOffset,localisationsInNm)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Called by run_training.m
%    Builds feature vectors for each localisations
%    Inputs:
%        folder - string, Folder in which the raw images are stored
%        pixelSize - number, Pixel size in nm
%        csvFile - string, path to .csv file where the localisations are stored
%	 imageFileExtension - string, extension of image files, e.g. '.tiff'
%        algorithms - string, depreciated, used to auto-load .csvinformation for certain algorithms. Set to 'Other'
%        ROI - matrix, depreciated, used to examine a sub-region of the image. Set to []
%        csvNumCols - int, Total number of columns in the .csv file
%        csvXCol - int, Column number in which the x co-ords are stored in the .csv file
%        csvYCol - int, Column number in which the y co-ords are stored in the .csv file
%        csvFCol - int, Column number in which the frame numbers are stored in the .csv file
%        pixelOffset - number, offset between pixel and localisation co-ordinates
%        localisationsInNm - boolean, sets whether the localisations are stored in nm or pixel co-ords
%    Outputs:
%        features - NX20 matrix, used as input features for classifier
%        rcfd - NX4 matrix, contains the row co-ord, column co-ord, frame number and directory number of each localisations
%        maxBright - number, brightest pixel encountered
%        minBright - number, dimmest pixel encountered
%        meanFeature - vector, average of observed features prior to PCA
%        V - matrix, transformation amtrix used to perform PCA and compress features to 20 dimensions
%        allOtherData - matrix, data from the .csv file other than position, frame number, etc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


	maxBright = -Inf;
	minBright = Inf;

	currentFolder = pwd;


	if not(exist('ROI'))
		ROI = [];
	end

	rcfd = [];

        allOtherData = [];

	N = 0;

	%build mean vector, covariance matirx, etc.
	for d=1:length(folder),
		cd(folder(d).name);

		[rcf, imgDir, patchWidth, patchSize, otherData] = get_rcf_etc(imageFileExtension, ROI, csvFile, algorithm, pixelSize,  csvNumCols, csvXCol, csvYCol, csvFCol, pixelOffset,localisationsInNm);
	
		
		N = N+size(rcf,1);

		h = waitbar(0,['Making features, folder ' ,num2str(d), ' of ', num2str(length(folder))]);

		for i=1:size(rcf,1),


			waitbar(i/size(rcf,1))
	
			%get feature
			[aPatch, priorPatch, posteriorPatch, maxBright,minBright] = getPatch(rcf(i,1), rcf(i,2), rcf(i,3), imgDir, patchSize, patchWidth, maxBright, minBright);
	
			feature = getfeature(aPatch, priorPatch, posteriorPatch, otherData(i,:));
	
			if not(exist('meanFeature')),
				meanFeature = zeros(size(feature));
				covFeatures = zeros(size(feature,1),size(feature,1));
			end
	
			%update mean, covar, estimate
			meanFeature = meanFeature + feature;
			covFeatures = covFeatures + feature*feature';
		end

		close(h);
		
		cd(currentFolder);
	end



	%PCA
	meanFeature = meanFeature/N;
	covFeatures = covFeatures/N;

	covFeatures = covFeatures - meanFeature*meanFeature';

	%get components
	[U S V] = svd(covFeatures);

	Df = 20;	%how many dimensions for our features

	V = V(:,1:Df);


	%compress using PCA

	features = zeros(N, Df);
	rcfd = zeros(N,4);
	j = 1;

	for d=1:length(folder),
		cd(folder(d).name);

		[rcf, imgDir, patchWidth, patchSize, otherData] = get_rcf_etc(imageFileExtension, ROI, csvFile, algorithm, pixelSize, csvNumCols, csvXCol, csvYCol, csvFCol, pixelOffset,localisationsInNm);
	
		h = waitbar(0,['Saving features, folder ' ,num2str(d), ' of ', num2str(length(folder))]);

		for i=1:size(rcf,1),

			waitbar(i/size(rcf,1))
	
			%get feature
			[aPatch, priorPatch, posteriorPatch] = getPatch(rcf(i,1), rcf(i,2), rcf(i,3), imgDir, patchSize, patchWidth, maxBright, minBright);
	
			feature = getfeature(aPatch, priorPatch, posteriorPatch, otherData(i,:));

			features(j,:) = (feature - meanFeature)'*V;
			rcfd(j,:) = [rcf(i,:),d];

			j = j+1;
		end

		if numel(otherData)>0,
			allOtherData = [allOtherData; otherData];
		end


		close(h);

		cd(currentFolder);
	end

end



function feature = getfeature(aPatch, priorPatch, posteriorpatch,otherDataRow);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Get raw feature from patch prior to PCA
%     input:
%         aPatch - 21X21 matrix, raw pixel values of patch
%         priorPatch - 21X21 matrix, raw pixel values of patch in image immediately preceeding
%         posteriorpatch - 21X21 matrix, raw pixel values of patch in image immediately following
%         otherDataRow - vector, any other data we want to go into the feature
%     output:
%         feature - vector, single feature made from all the input data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	h = fspecial('gaussian', size(aPatch,1), 1/3*((size(aPatch,1)-1)/2));

	aPatch = aPatch.*h;
	priorPatch = priorPatch.*h;
	posteriorpatch = posteriorpatch.*h;

	feature = log10(max(realmin, abs([aPatch(:); priorPatch(:); posteriorpatch(:); otherDataRow'])));

end


function [rcf, imgDir, patchWidth, patchSize, otherData] = get_rcf_etc(imageFileExtension, ROI, csvFile, algorithm, pixelSize, csvNumCols, csvXCol, csvYCol, csvFCol, pixelOffset,localisationsInNm)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Get rcf, patch size, etc for a particular set of images in a directory
%    Inputs:
%        pixelSize - number, Pixel size in nm
%        csvFile - string, path to .csv file where the localisations are stored
%	 imageFileExtension - string, extension of image files, e.g. '.tiff'
%        ROI - matrix, depreciated, used to examine a sub-region of the image. Set to []
%        csvNumCols - int, Total number of columns in the .csv file
%        csvXCol - int, Column number in which the x co-ords are stored in the .csv file
%        csvYCol - int, Column number in which the y co-ords are stored in the .csv file
%        csvFCol - int, Column number in which the frame numbers are stored in the .csv file
%        pixelOffset - number, offset between pixel and localisation co-ordinates
%        localisationsInNm - boolean, sets whether the localisations are stored in nm or pixel co-ords
%    Outputs:
%        rcf - NX3 matrix, contains the row co-ord, column co-ord, frame number
%        imgDir - struct, list of all file names of image files in folder of interest
%        patchWidth - distance from edge of patch to central pixel
%        patchSize - distance from one edge of patch to the opposite
%        otherData - other data int he .csv file, not including the localisation co-ords or frame number
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



	imgDir = dir(imageFileExtension);%get images
	patchWidth = 10;
	patchSize = ceil(patchWidth)*2 +1;

	%read in localisations + convert to pixel co-ords
	disp(csvFile);
	%check for zero bytes file
	dirCSV = dir(csvFile);
	if dirCSV.bytes==0,
		rcf = zeros(0,4);
		otherData = zeros(0,0);
  	else
        	try 
    			CSV = csvread(csvFile);
        	catch ME
            		fid  = fopen(csvFile);
            		formatstring = repmat('%f',1,csvNumCols);
            		CSVcell = textscan(fid,formatstring,'Delimiter',',','HeaderLines',1);
            		fclose(fid);
            		CSV = [];
            		for c=1:length(CSVcell),
            		   CSV = [CSV, CSVcell{c}]; 
            		end
        	end

	        rcf = [CSV(:,csvYCol), CSV(:,csvXCol), CSV(:,csvFCol)];
        	if localisationsInNm,
            		rcf(:,1:2) = rcf(:,1:2)/pixelSize;
        	end
        	rcf(:,1:2) = rcf(:,1:2) + pixelOffset;
        
        	otherCol = [1:csvNumCols];
        	otherCol([csvYCol; csvXCol; csvFCol]) = [];
        	otherData = CSV(:, otherCol);
	
		if not(isempty(ROI)),
			reject = find( (rcf(:,1) < ROI(1,1)) | (rcf(:,1) > ROI(1,2)) | (rcf(:,2) < ROI(2,1)) | (rcf(:,2) > ROI(2,2)) | (rcf(:,3) < ROI(3,1)) | (rcf(:,3) > ROI(3,2)) );
			rcf(reject,:) = [];
			otherData(reject,:) = [];
		end
	
		patchWidth = 10;
	
		patchSize = ceil(patchWidth)*2 +1;
	
		%strip out any localisations too close to the edge of the image
		img = imread(imgDir(1).name);
		[nRows, nCols] = size(img);
		imgMargin = ceil(patchWidth*sqrt(2))+2;
	
		reject = find((rcf(:,1)-imgMargin < 1)|(rcf(:,1)+imgMargin > nRows)|(rcf(:,2)-imgMargin < 1)|(rcf(:,2)+imgMargin > nCols)|(rcf(:,3) <2)|(rcf(:,3) >=length(imgDir)));
	
		rcf(reject,:) = [];
		otherData(reject,:) = [];
	end


end




function [aPatch, priorPatch, posteriorPatch, maxBright, minBright] = getPatch(r,c,f,imgDir,patchSize,patchWidth,maxBright,minBright)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    For a localisation at (r,c) in image f, build the current pixel patch, prior and posterior patches, and update record of max/min brightness encoutered so far
%    input:
%        r - number, row co-ordinate of localisation
%        c - number, column co-ordinate of localisation
%        f - int, frame number of localisation
%        imgDir - struct, list of all image files names in folder
%        patchSize - distance from one edge of patch to the opposite
%        patchWidth - distance from edge of patch to central pixel
%        maxBright - number, brightest pixel encountered
%        minBright - number, dimmest pixel encountered
%    output:
%        aPatch - matrix, rotation corrected pixel values of patch centered at (r,c) in image number f
%        priorPatch - matrix, rotation corrected pixel values of patch centered at (r,c) in image number f-1
%        posteriorPatch - matrix, rotation corrected pixel values of patch centered at (r,c) in image number f+1
%        maxBright - number, brightest pixel encountered
%        minBright  - number, dimmest pixel encountered     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	persistent fCurrent;
	persistent img;
	persistent priorImg;
	persistent posteriorImg;

	if isempty(fCurrent),
		fCurrent = -1;
	end

	if fCurrent~=f,
		
		img = double(imread(imgDir(f).name));
		maxBright = max(max(img(:)), maxBright);
		minBright = min(min(img(:)), minBright);
		

		priorImg = double(imread(imgDir(f-1).name));
		
		
		posteriorImg = double(imread(imgDir(f+1).name));
			
		fCurrent = f;
	end


	charScale = 0.55;%if we want a constant amount of blur, this this is the place to set it. otherwise skip this part

	[aPatch, theta, charScale] = orientated_clipped_patch(img, r, c, patchWidth, charScale);
	
	priorPatch = orientated_clipped_patch(priorImg, r, c, patchWidth, charScale, theta);
		
	posteriorPatch = orientated_clipped_patch(posteriorImg, r, c, patchWidth, charScale, theta);

end





function [imgPatch T charScale] = orientated_clipped_patch(img, r, c, patchWidth, charScale, T)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Create an image patch at point (r,c)
%    Patch is rotated by T, or if T is left out, so as to normalise the direction of maximum gradient at center of the patch (rotation invariance)
%    Patch is blurred by gaussian of scale charScale. Used to intorduce a characteristic scale below which features are not distinguishable,
%    to help build in robustness to ssmall displacements, etc
%    inputs:
%        img - matrix, image pixel values
%        r - number, row co-ordinate
%        c - number column co-ordinate
%        patchWidth - distance from edge of patch to central pixel
%        charScale - number, characteristic scale of patch (which controls the degree of blur applied)
%        T - number, rotation to apply to patch 
%    outputs:
%        imgPatch - matrix, iamge aptch centered at (r,c), rotated and blurred
%        T  - number, rotation applied to patch. if this was not an input, it is now set to nromalise image gradient 
%        charScale - number, characteristic scale of blur applied to patch.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



	%start with abig patch to rotate corectly, then clip it down to a small patch
	bigPatchWidth = ceil(patchWidth*sqrt(2))+2;

	

	bigPatchUpRight   = img((ceil(r)-bigPatchWidth):(ceil(r)+bigPatchWidth), (ceil(c)-bigPatchWidth):(ceil(c)+bigPatchWidth));
	bigPatchDownRight = img((ceil(r)-bigPatchWidth):(ceil(r)+bigPatchWidth), (floor(c)-bigPatchWidth):(floor(c)+bigPatchWidth));
	bigPatchUpleft    = img((floor(r)-bigPatchWidth):(floor(r)+bigPatchWidth), (ceil(c)-bigPatchWidth):(ceil(c)+bigPatchWidth));
	bigPatchDownleft  = img((floor(r)-bigPatchWidth):(floor(r)+bigPatchWidth), (floor(c)-bigPatchWidth):(floor(c)+bigPatchWidth));

	wUR = mod(r,1)*mod(c,1);
	wDR = (1-mod(r,1))*mod(c,1);
	wUL = mod(r,1)*(1-mod(c,1));
	wDL = (1-mod(r,1))*(1-mod(c,1));

	bigPatch = wUR*bigPatchUpRight + wDR*bigPatchDownRight + wUL*bigPatchUpleft + wDL*bigPatchDownleft;


	
	if not(exist('charScale')),
	%get smallest scalespace maxima. Really ought to return *all* scalespace maxima, but smallest seems best for now

		scaleSpace = [0.1:0.1:0.1*(ceil(patchWidth)*2 +1)];
		dog = zeros(size(scaleSpace));
		for sigamIdx = 1:length(scaleSpace),

			f = fspecial('gaussian', size(bigPatch,1), scaleSpace(sigamIdx));
			dog(sigamIdx) = sum(sum(f.*bigPatch));
		end

		dog = dog(1:(end-1)) - dog(2:end); 
		scaleSpace = scaleSpace(1:(end-1)); 
		dog = dog.*scaleSpace;

		localmax = logical(ones(size(dog)));
		localmax(1:(end-1)) = localmax(1:(end-1))&(dog(1:(end-1))>=dog(2:end));
		localmax(2:end) = localmax(2:end)&(dog(1:(end-1))<=dog(2:end));

		charScale = scaleSpace(find(localmax));
		charScale = charScale(1);%first scale only
	end

	f = fspecial('gaussian', size(bigPatch,1), charScale);
	bigPatch = conv2(bigPatch, f, 'same');


	%get rotation angle
	if not(exist('T')),
		dy = conv2(bigPatch, [1;0;-1], 'same');
		dx = conv2(bigPatch, [1,0,-1], 'same');
		theta = atan(dy./(dx+double(dx==0)*realmin));
		theta = theta((bigPatchWidth+1-patchWidth+2):(bigPatchWidth+1+patchWidth-2) , (bigPatchWidth+1-patchWidth+2):(bigPatchWidth+1+patchWidth-2))*180/pi;	%get rid of edges as they're weird
		theta = theta(:);
		thetaPrecision = 360/16;
		theta = round(theta/thetaPrecision)*thetaPrecision;
		T = mode(theta);
	end


	%and rotate
	bigPatch = imrotate(bigPatch, T,'bilinear','crop');



	%and crop
	imgPatch = bigPatch( (bigPatchWidth+1-patchWidth):(bigPatchWidth+1+patchWidth) , (bigPatchWidth+1-patchWidth):(bigPatchWidth+1+patchWidth));



end
